{ ****************************************************************** }
{                                                                    }
{   VCL component TSPparse                                           }
{                                                                    }
{   Non-Visual Component to split a String.                          }
{                                                                    }
{   Copyright  2000 by Stefan Petersen (sp531561@t-online.de)       }
{                                                                    }
{ ****************************************************************** }

unit SPparse;

interface

uses WinTypes, WinProcs, Messages, SysUtils, Classes, Controls;

{ Unit-wide declarations }
{ Type }
 { . . . }
{ Var }
 { . . . }

type
  TSPparse = class(TComponent)
    private
      { Private fields of TSPparse }
        { Storage for property AboutComponent }
        FAboutComponent : String;
        { Storage for property AboutVersion }
        FAboutVersion : String;
        { Storage for property Character }
        FCharacter : String;
        { Storage for property Found }
        FFound : Word;
        { Storage for property IsSpaceAfterCharacter }
        FIsSpaceAfterCharacter : Boolean;
        { Storage for property SearchText }
        FSearchText : String;
        { Storage for property Start }
        FStart : Word;
        { Storage for property Stop }
        FStop : Word;
        { Storage for property Words }
        FWords : TStrings;
        { Pointer to application's OnWordFound handler, if any }
        FOnWordFound : TNotifyEvent;

      { Private methods of TSPparse }
        { Method to set variable and property values and create objects }
        procedure AutoInitialize;
        { Method to free any objects created by AutoInitialize }
        procedure AutoDestroy;
        { Read method for property AboutComponent }
        function GetAboutComponent : String;
        { Read method for property AboutVersion }
        function GetAboutVersion : String;
        { Read method for property Character }
        function GetCharacter : String;
        { Write method for property Character }
        procedure SetCharacter(Value : String);
        { Read method for property Found }
        function GetFound : Word;
        { Write method for property Found }
        procedure SetFound(Value : Word);
        { Read method for property IsSpaceAfterCharacter }
        function GetIsSpaceAfterCharacter : Boolean;
        { Write method for property IsSpaceAfterCharacter }
        procedure SetIsSpaceAfterCharacter(Value : Boolean);
        { Read method for property SearchText }
        function GetSearchText : String;
        { Write method for property SearchText }
        procedure SetSearchText(Value : String);
        { Read method for property Start }
        function GetStart : Word;
        { Write method for property Start }
        procedure SetStart(Value : Word);
        { Read method for property Stop }
        function GetStop : Word;
        { Write method for property Stop }
        procedure SetStop(Value : Word);
        { Write method for property Words }
        procedure SetWords(Value : TStrings);

    protected
      { Protected fields of TSPparse }
        property Start : Word read GetStart write SetStart;
        property Stop : Word read GetStop write SetStop;

      { Protected methods of TSPparse }
        { Method to generate OnWordFound event }
        procedure WordFound(Sender : TObject); virtual;
        { Loaded-Procedure of the Component }
        procedure Loaded; override;

    public
      { Public fields and properties of TSPparse }

      { Public methods of TSPparse }
        { Constructor of the Component }
        constructor Create(AOwner: TComponent); override;
        { Destructor of the Component }
        destructor Destroy; override;
        { The Execute-Function of the Component, (True if successfull) }
        function Execute : Boolean;

    published
      { Published properties of TSPparse }
        { Event is fired when a Word is found }
        property OnWordFound : TNotifyEvent read FOnWordFound write FOnWordFound;
        { About the Component }
        property AboutComponent : String
             read GetAboutComponent write FAboutComponent;
        { The Version and Date of the Component }
        property AboutVersion : String
             read GetAboutVersion write FAboutVersion;
        { The Character, where to split the String }
        property Character : String read GetCharacter write SetCharacter;
        { The amount of Words the SearchText was split into }
        property Found : Word read GetFound write SetFound default 0;
        { Set to True, if there is a space after the Split-Character }
        property IsSpaceAfterCharacter : Boolean
             read GetIsSpaceAfterCharacter write SetIsSpaceAfterCharacter
             default False;
        { The String, which will be parsed }
        property SearchText : String read GetSearchText write SetSearchText;
        { Words is the result after the SearchText was split }
        property Words : TStrings read FWords write SetWords;

  end;

procedure Register;

implementation

{ Unit-wide declarations }
{ Type }
 { . . . }
{ Var }
 { . . . }

procedure Register;
begin
     { Register TSPparse with MyLib as its
       default page on the Delphi component palette }
     RegisterComponents('MyLib', [TSPparse]);
end;

{ Method to set variable and property values and create objects }
procedure TSPparse.AutoInitialize;
begin
     FCharacter := ';';
     FFound := 0;
     FIsSpaceAfterCharacter := False;
     FSearchText := '';
     FWords := TStringList.Create;
end; { of AutoInitialize }

{ Method to free any objects created by AutoInitialize }
procedure TSPparse.AutoDestroy;
begin
     FWords.Free;
end; { of AutoDestroy }

{ Read method for property AboutComponent }
function TSPparse.GetAboutComponent : String;
Begin
 Result := 'Copyright  2000 by Stefan Petersen (sp531561@t-online.de)';
End;

{ Read method for property AboutVersion }
function TSPparse.GetAboutVersion : String;
Begin
 Result := '1.1 (14.Nov.2000)';
End;

{ Read method for property Character }
function TSPparse.GetCharacter : String;
Begin
 If Length (FCharacter) > 1
 Then
 FCharacter := FCharacter [1];
 Result := FCharacter;
End;

{ Write method for property Character }
procedure TSPparse.SetCharacter(Value : String);
Begin
 FCharacter := Value;
End;

{ Read method for property Found }
function TSPparse.GetFound : Word;
Begin
 Result := FFound;
End;

{ Write method for property Found }
procedure TSPparse.SetFound(Value : Word);
Begin
 FFound := Value;
End;

{ Read method for property IsSpaceAfterCharacter }
function TSPparse.GetIsSpaceAfterCharacter : Boolean;
Begin
 Result := FIsSpaceAfterCharacter;
End;

{ Write method for property IsSpaceAfterCharacter }
procedure TSPparse.SetIsSpaceAfterCharacter(Value : Boolean);
Begin
 FIsSpaceAfterCharacter := Value;
End;

{ Read method for property SearchText }
function TSPparse.GetSearchText : String;
Begin
 Result := FSearchText;
End;

{ Write method for property SearchText }
procedure TSPparse.SetSearchText(Value : String);
Begin
 FSearchText := Value;
End;

{ Read method for property Start }
function TSPparse.GetStart : Word;
begin
 Result := FStart;
end;

{ Write method for property Start }
procedure TSPparse.SetStart(Value : Word);
begin
 FStart := Value;
end;

{ Read method for property Stop }
function TSPparse.GetStop : Word;
begin
 Result := FStop;
end;

{ Write method for property Stop }
procedure TSPparse.SetStop(Value : Word);
begin
 FStop := Value;
end;

{ Write method for property Words }
procedure TSPparse.SetWords(Value : TStrings);
Begin
 { Use Assign method because TStrings is an object type
 and FWords has been created. }
 FWords.Assign(Value);
End;

{ Method to generate OnWordFound event }
procedure TSPparse.WordFound(Sender : TObject);
Begin
 { Has the application assigned a method to the event, whether
 via the Object Inspector or a run-time assignment?  If so,
 execute that method }
 If Assigned(FOnWordFound) Then
 FOnWordFound(Sender);
End;

{ Constructor of the Component }
constructor TSPparse.Create(AOwner: TComponent);
Begin
 Inherited Create(AOwner);
 AutoInitialize;
 { Code to perform other tasks when the component is created }
End;

{ Destructor of the Component }
destructor TSPparse.Destroy;
Begin
 AutoDestroy;
 Inherited Destroy;
End;

{ The Execute-Function of the Component, (True if successfull) }
function TSPparse.Execute : Boolean;
Var
 i                  : Word;
Begin
 { Perform the component operation }
 If Length(FSearchText) = 0 Then
  Begin
   Result := False;
   Exit;
  End;
 { Set start to 1, }
 Start := 1;
 { and found to 0 }
 Found := 0;
 { Clear the ListBox }
 FWords.Clear;
 { Add a FCharacter at the end of FSearchText, }
 { otherwise single Characters would not be detected as a word }
 If FSearchText[Length(FSearchText)] <> FCharacter Then
  FSearchText := FSearchText + FCharacter;
 { We'll search through the entire string }
 For i := 1 To Length(FSearchText) Do
  Begin
   { If a character is a FCharacter then }
   If Copy(FSearchText, i, 1) = FCharacter Then
    Begin
     { Set stop to the position - 1 (we don't want the FCharacter) }
     Stop := i - 1;
     { Add word to the ListBox }
     FWords.Add(Copy(FSearchText, Start, Stop - Start + 1));
     { Set new start (next word) }
     { If you use spaces after the FCharacter's you'll have to change }
     { the value to i+spaces+1, where spaces is the number of white spaces }
     { you have after each FCharacter }
     If FIsSpaceAfterCharacter Then
      Start := i + 2
     Else
      Start := i + 1;
     { Increase the found variable if it's not the final character }
     If i <> Length(FSearchText) Then
      Begin
       Inc(FFound);
       If Assigned(FOnWordFound) Then
        FOnWordFound(Self);
      End;
    End;
  End;
 { We're not sure there's a ; after the last word }
 { If the last character is NOT a FCharacter and }
 { if there's text in FSearchText }
 If (Copy(FSearchText, Length(FSearchText) - 1, 1) <> FCharacter) And
  (FSearchText <> '') Then
  Begin
   { Add word to ListBox }
   FWords.Add(Copy(FSearchText, Start, Length(FSearchText) - Start + 1));
   { Increase found }
   Inc(FFound);
   If Assigned(FOnWordFound) Then
    FOnWordFound(Self);
  End;
 { Perhaps there's only one word, and no FCharacter's }
 If (FFound = 0) And (FSearchText <> '') Then
  Begin
   If Copy(FSearchText, Length(FSearchText) - 1, 1) <> FCharacter Then
    Begin
     { Then we'll add it to the ListBox, and set found to 1 }
     FWords.Add(Copy(FSearchText, Start, Length(FSearchText) - Start + 1));
     FFound := 1;
     If Assigned(FOnWordFound) Then
      FOnWordFound(Self);
    End;
  End;
 { Return True if the operation was successful, False otherwise }
 Result := True;
End;

{ Loaded-Procedure of the Component }
procedure TSPparse.Loaded;
Begin
 Inherited Loaded;
 { Perform any component setup that depends }
 { on the property values having been set. }
End;


end.
